"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const path = require("path");
const decorators_1 = require("./common/decorators");
class NodePackageManager {
    constructor($fs, $hostInfo, $errors, $childProcess, $logger) {
        this.$fs = $fs;
        this.$hostInfo = $hostInfo;
        this.$errors = $errors;
        this.$childProcess = $childProcess;
        this.$logger = $logger;
    }
    install(packageName, pathToSave, config) {
        return __awaiter(this, void 0, void 0, function* () {
            if (config.disableNpmInstall) {
                return;
            }
            if (config.ignoreScripts) {
                config["ignore-scripts"] = true;
            }
            let packageJsonPath = path.join(pathToSave, "package.json");
            let jsonContentBefore = this.$fs.readJson(packageJsonPath);
            let flags = this.getFlagsString(config, true);
            let params = ["install"];
            const isInstallingAllDependencies = packageName === pathToSave;
            if (!isInstallingAllDependencies) {
                params.push(packageName);
            }
            params = params.concat(flags);
            let cwd = pathToSave;
            const etcDirectoryLocation = path.join(cwd, "etc");
            const etcExistsPriorToInstallation = this.$fs.exists(etcDirectoryLocation);
            if (config.path) {
                let relativePathFromCwdToSource = "";
                if (config.frameworkPath) {
                    relativePathFromCwdToSource = path.relative(config.frameworkPath, pathToSave);
                    if (this.$fs.exists(relativePathFromCwdToSource)) {
                        packageName = relativePathFromCwdToSource;
                    }
                }
            }
            try {
                let spawnResult = yield this.$childProcess.spawnFromEvent(this.getNpmExecutableName(), params, "close", { cwd, stdio: "inherit" });
                if (!etcExistsPriorToInstallation) {
                    this.$fs.deleteDirectory(etcDirectoryLocation);
                }
                if (isInstallingAllDependencies) {
                    return null;
                }
                params = params.concat(["--json", "--dry-run", "--prefix", cwd]);
                spawnResult = yield this.$childProcess.spawnFromEvent(this.getNpmExecutableName(), params, "close");
                return this.parseNpmInstallResult(spawnResult.stdout);
            }
            catch (err) {
                if (err.message && err.message.indexOf("EPEERINVALID") !== -1) {
                    this.$logger.warn(err.message);
                }
                else {
                    this.$fs.writeJson(packageJsonPath, jsonContentBefore);
                    throw err;
                }
            }
        });
    }
    uninstall(packageName, config, path) {
        return __awaiter(this, void 0, void 0, function* () {
            const flags = this.getFlagsString(config, false);
            return this.$childProcess.exec(`npm uninstall ${packageName} ${flags}`, { cwd: path });
        });
    }
    search(filter, config) {
        return __awaiter(this, void 0, void 0, function* () {
            const flags = this.getFlagsString(config, false);
            return this.$childProcess.exec(`npm search ${filter.join(" ")} ${flags}`);
        });
    }
    view(packageName, config) {
        return __awaiter(this, void 0, void 0, function* () {
            const wrappedConfig = _.extend({}, config, { json: true });
            let flags = this.getFlagsString(wrappedConfig, false);
            let viewResult;
            try {
                viewResult = yield this.$childProcess.exec(`npm view ${packageName} ${flags}`);
            }
            catch (e) {
                this.$errors.failWithoutHelp(e);
            }
            return JSON.parse(viewResult);
        });
    }
    getNpmExecutableName() {
        let npmExecutableName = "npm";
        if (this.$hostInfo.isWindows) {
            npmExecutableName += ".cmd";
        }
        return npmExecutableName;
    }
    getFlagsString(config, asArray) {
        let array = [];
        for (let flag in config) {
            if (flag === "global") {
                array.push(`--${flag}`);
                array.push(`${config[flag]}`);
            }
            else if (config[flag]) {
                if (flag === "dist-tags" || flag === "versions") {
                    array.push(` ${flag}`);
                    continue;
                }
                array.push(`--${flag}`);
            }
        }
        if (asArray) {
            return array;
        }
        return array.join(" ");
    }
    parseNpmInstallResult(npmInstallOutput) {
        const originalOutput = JSON.parse(npmInstallOutput);
        const name = _.head(_.keys(originalOutput.dependencies));
        const dependency = _.pick(originalOutput.dependencies, name);
        return {
            name,
            originalOutput,
            version: dependency[name].version
        };
    }
}
__decorate([
    decorators_1.exported("npm")
], NodePackageManager.prototype, "install", null);
__decorate([
    decorators_1.exported("npm")
], NodePackageManager.prototype, "uninstall", null);
__decorate([
    decorators_1.exported("npm")
], NodePackageManager.prototype, "search", null);
__decorate([
    decorators_1.exported("npm")
], NodePackageManager.prototype, "view", null);
exports.NodePackageManager = NodePackageManager;
$injector.register("npm", NodePackageManager);
